'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.restful = exports.delete_ = exports.update = exports.updateOrCreate = exports.create = exports.retrieve = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Default predicate used to valid / filters some entities.
 *
 * @param  {Object} entity   An entity
 * @param  {Object} params   Params or body of the query
 * @return {Boolean}
 */
var defaultPredicate = function defaultPredicate(entity, params) {
  var idKey = arguments.length <= 2 || arguments[2] === undefined ? 'id' : arguments[2];

  return Number(entity[idKey]) === Number(params[idKey]);
};

/**
 * Retrieve entities. Typically, handle GET verbs.
 *
 * From: https://spring.io/understanding/REST
 * "Retrieve information. GET requests must be safe and idempotent, meaning
 * regardless of how many times it repeats with the same parameters, the results
 * are the same. They can have side effects, but the user doesn't expect them,
 * so they cannot be critical to the operation of the system. Requests can
 * also be partial or conditional."
 *
 * @param  {Mixed} state          Current state
 * @return {Mixed}                New state
 */
var retrieve = exports.retrieve = function retrieve(state) {
  var idKey = arguments.length <= 1 || arguments[1] === undefined ? 'id' : arguments[1];
  return _lodash2.default.sortBy(state, idKey);
};

/**
 * Create an entity. Typically, handle POST verbs.
 *
 * From: https://spring.io/understanding/REST
 * "Request that the resource at the URI do something with the provided entity.
 * Often POST is used to create a new entity, but it can also be used to update
 * an entity."
 *
 * @param  {Array} state          Current state
 * @param  {Object} body          Body of the request
 * @param  {Object} SchemaObject  Schema of the new entity
 * @param  {String} idKey         Name of the id key
 * @return {Array}                New state
 */
var create = exports.create = function create(state, body, SchemaObject) {
  var idKey = arguments.length <= 3 || arguments[3] === undefined ? 'id' : arguments[3];

  if (body[idKey]) {
    throw new Error('body should not provide an id when creating an entity.');
  }

  var idValue = _lodash2.default.max(state.map(function (entity) {
    return entity[idKey];
  })) + 1;

  var entity = new SchemaObject(_extends({}, body, _defineProperty({}, idKey, idValue)));

  return _lodash2.default.sortBy([].concat(_toConsumableArray(state), [entity.toObject()]), idKey);
};

/**
 * Update or create an entity. Typically, handle PUT verbs.
 *
 * From: https://spring.io/understanding/REST
 * "Store an entity at a URI. PUT can create a new entity or update an existing
 * one. A PUT request is idempotent. Idempotency is the main difference between
 * the expectations of PUT versus a POST request."
 *
 * Note: PUT replaces an existing entity. If only a subset of data elements
 * are provided, the rest will be replaced with empty or null.
 *
 * @param  {Object} state        Current state
 * @param  {Object} params       Params of the query
 * @param  {Object} body         Body of the query
 * @param  {Object} SchemaObject Schema of the new entity
 * @param  {Function} predicate  Predicate function
 * @param  {String} idKey        Name of the id key
 * @return {Object}              New state
 */
var updateOrCreate = exports.updateOrCreate = function updateOrCreate(state, params, body, SchemaObject) {
  var idKey = arguments.length <= 4 || arguments[4] === undefined ? 'id' : arguments[4];
  var predicate = arguments.length <= 5 || arguments[5] === undefined ? defaultPredicate : arguments[5];

  if (!params[idKey]) {
    throw new Error('Params should provide an id when updating an entity.');
  }

  var existingEntity = state.filter(function (entity) {
    return predicate(entity, params, idKey);
  }).pop();

  // create a new entity with a defined idKey
  if (!existingEntity) {
    var entity = new SchemaObject(_extends({}, body, _defineProperty({}, idKey, params[idKey])));

    return [].concat(_toConsumableArray(state), [entity.toObject()]);
  }

  // update entity
  var updatedEntity = _extends({}, existingEntity, body);

  var sanitizedEntity = new SchemaObject(updatedEntity).toObject();

  // set to null the properties that are not provided in body
  _lodash2.default.difference(Object.keys(existingEntity), Object.keys(body)).map(function (key) {
    if (key === idKey) {
      // dont set to null the idKey
      return;
    }
    sanitizedEntity[key] = null;
  });

  var newState = state.slice(0);
  _lodash2.default.remove(newState, function (entity) {
    return predicate(entity, params, idKey);
  });

  return _lodash2.default.sortBy([].concat(_toConsumableArray(newState), [sanitizedEntity]), idKey);
};

/**
 * Update an entity. Typically, handle PATCH verbs.
 *
 * From: https://spring.io/understanding/REST
 * Update only the specified fields of an entity at a URI. A PATCH request is
 * idempotent. Idempotency is the main difference between the expectations of
 * PATCH versus a POST request.
 */
var update = exports.update = function update(state, params, body, SchemaObject) {
  var idKey = arguments.length <= 4 || arguments[4] === undefined ? 'id' : arguments[4];
  var predicate = arguments.length <= 5 || arguments[5] === undefined ? defaultPredicate : arguments[5];

  if (!params[idKey]) {
    throw new Error('Params should provide an id when updating an entity.');
  }

  var existingEntity = state.filter(function (entity) {
    return predicate(entity, params, idKey);
  }).pop();

  if (!existingEntity) {
    return state;
  }

  // update entity
  var updatedEntity = _extends({}, existingEntity, body);

  var sanitizedEntity = new SchemaObject(updatedEntity);

  var newState = state.slice(0);
  _lodash2.default.remove(newState, function (entity) {
    return predicate(entity, params, idKey);
  });

  return _lodash2.default.sortBy([].concat(_toConsumableArray(newState), [sanitizedEntity.toObject()]), idKey);
};

/**
 * Delete an entity. Typically, handle DELETE verbs.
 *
 * From: https://spring.io/understanding/REST
 * Request that a resource be removed; however, the resource does not have to
 * be removed immediately. It could be an asynchronous or long-running request.
 *
 * @param  {Array} state          Current state
 * @param  {Object} params        Params of the request
 * @param  {Function} predicate   Predicate function
 * @return {Array}                New state
 */
var delete_ = exports.delete_ = function delete_(state, params) {
  var idKey = arguments.length <= 2 || arguments[2] === undefined ? 'id' : arguments[2];
  var predicate = arguments.length <= 3 || arguments[3] === undefined ? defaultPredicate : arguments[3];

  var stateCopy = state.slice(0);
  _lodash2.default.remove(stateCopy, function (entity) {
    return predicate(entity, params, idKey);
  });
  return stateCopy;
};

/**
 * Return an object compatible to redux-act.createReducer that defines the
 * whole state management for a Restful API.
 *
 * Note:
 * Like any reducers, it handles only the state,
 * not the output that concerns selectors.
 *
 * @param  {Object} actions           Actions
 * @param  {Object} SchemaObject      Schema of the new entity
 * @param  {Function} predicate       Predicate function
 * @return {Object}                   Reducer definition
 */
var restful = exports.restful = function restful(actions, SchemaObject) {
  var _ref5;

  var predicate = arguments.length <= 2 || arguments[2] === undefined ? defaultPredicate : arguments[2];
  var idKey = arguments.length <= 3 || arguments[3] === undefined ? 'id' : arguments[3];

  return _ref5 = {}, _defineProperty(_ref5, actions.get, function (state) {
    return retrieve(state, idKey);
  }), _defineProperty(_ref5, actions.post, function (state, _ref) {
    var body = _ref.body;
    return create(state, body, SchemaObject, idKey);
  }), _defineProperty(_ref5, actions.put, function (state, _ref2) {
    var params = _ref2.params;
    var body = _ref2.body;
    return updateOrCreate(state, params, body, SchemaObject, idKey, predicate);
  }), _defineProperty(_ref5, actions.patch, function (state, _ref3) {
    var params = _ref3.params;
    var body = _ref3.body;
    return update(state, params, body, SchemaObject, idKey, predicate);
  }), _defineProperty(_ref5, actions.delete, function (state, _ref4) {
    var params = _ref4.params;
    return delete_(state, params, idKey, predicate);
  }), _ref5;
};