'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extendRows = exports.selectRow = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _reselect = require('reselect');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Select one or severals rows from `selector` if the key `key` is equal to
 * `value`.
 *
 * @param  {Function}         Selector function (from reselect)
 * @param  {String}           key
 * @param  {String | Number}  value
 * @return {Object | Array}
 */
/* eslint no-console: 0 */

var selectRow = exports.selectRow = function selectRow(selector, key, value) {
  if (_lodash2.default.isUndefined(selector) || _lodash2.default.isUndefined(key) || _lodash2.default.isUndefined(value)) {
    return function () {
      return 'Bad arguments for `selectRow`';
    };
  }

  return (0, _reselect.createSelector)(selector(), function (rows) {
    var result = _lodash2.default.groupBy(rows, key)[value];
    if (_lodash2.default.isArray(result) && result.length === 1) {
      result = result.pop();
    }
    return result;
  });
};

/**
 * Return the results of `selector1` extended with results of `selector2`
 * saved in `destKey` according to the predicate function (or string).

 * @param  {Function} selector1           Selector function
 * @param  {String} key1                  Key to compare in the first collection
 * @param  {Function} selector2           Selector function
 * @param  {String} ke2                   Key to compare in the second collection
 * @param  {String} destKey               The destination key in the first collection
 * @param  {Function | String} predicate  Optional custom comparison function
 * @return {Array}
 */
var extendRows = exports.extendRows = function extendRows(selector1, key1, selector2, key2, destKey, predicate) {
  if (!_lodash2.default.isFunction(selector1) || !_lodash2.default.isFunction(selector2) || !_lodash2.default.isString(key1) || !_lodash2.default.isString(key2) || !_lodash2.default.isString(destKey) || predicate !== undefined && !_lodash2.default.isFunction(predicate)) {
    return function () {
      return 'Bad arguments for `extendRows`';
    };
  }

  var predicateFunction = predicate;

  if (_lodash2.default.isUndefined(predicateFunction)) {
    predicateFunction = function (row1, key1_, row2, key2_) {
      return Number(row1[key1_]) === Number(row2[key2_]);
    };
  }

  return (0, _reselect.createSelector)(selector1(), selector2(), function (rows1, rows2) {
    return rows1.map(function (row1) {
      var clonedRow1 = Object.assign({}, row1);
      clonedRow1[destKey] = rows2.filter(function (row2) {
        return predicateFunction(row1, key1, row2, key2);
      }).pop();
      return clonedRow1;
    });
  });
};